
//
// Example of use of Object Tango API
//
// The example consists of three classes : class Ex, class Icon1, class Icon2
// class Ex is the class extending Applet, classes Icon1 and Icon2 are implementing
// TObject interface, which means that objects of those classes should indicate
// collaborative behavior.
//
// The interface to the program consists of two radio buttons, and the drawing area.
// Radio buttons determine which type of icon will be added if the user pressed mouse
// button over the empty space in the draw area. The added icon will automatically
// appear on all instances of the application connected to the session.
// The user may also drag the icon, these changes are also reflected on all instances.
//
// @author Piotr Sokolowski
// @version $Revision: 1.1 $
// 

package webwisdom.examples;

import webwisdom.tango.*;
import webwisdom.tango.object.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.applet.*;


//
// Class Icon1 represents an icon on the screen. The Icon1 objects will be automaticly recreated
// on all clients in the session.
//
// This class must be public so that TNode class could recreate it on other clients.
//
public class Icon1 extends Component implements TObject
{
  TNode tn;
  int x;
  int y;
  int size=15;
  int lastmx;
  int lastmy;
  
  public Icon1(TNode tn,Point p)
    // creates icon at point p. The icon will be created on all instances in the session
    {
      this.tn=tn;
      x=p.x;
      y=p.y;
      ((Container)tn.getTObject("drawpanel")).add(this);
      setBounds(x,y,size,size);
      tn.add(this);
      enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK+AWTEvent.MOUSE_EVENT_MASK);
    }
  public Icon1(TNode tn,DataInputStream dis) throws Exception
    // this constructor is called by TNode object, when the class is created on the remote
    // instance
    {
      
      this.tn=tn;
      ((Container)tn.getTObject("drawpanel")).add(this);
      update(dis);
      enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK+AWTEvent.MOUSE_EVENT_MASK);
    }
  public void paint(Graphics g)
    {
      g.setColor(Color.red);
      g.fillRect(0,0,getSize().width,getSize().height);
    }
  
  public void processMouseEvent(MouseEvent me)
    // remembering initial mouse position for drag calculation
    {
      lastmx=me.getX();
      lastmy=me.getY();
      super.processMouseEvent(me);
    }
  public void processMouseMotionEvent(MouseEvent me)
    // drag processing
    {
      if(me.getID()==MouseEvent.MOUSE_DRAGGED)
	{
	  x=x+me.getX()-lastmx;
	  y=y+me.getY()-lastmy;
	  setBounds(x,y,size,size);
	  getParent().repaint();
	  send(); // sends the changes to other clients
	}
      super.processMouseMotionEvent(me);
    }
 
  public void send()
    // sends the position information to other clients
    {
      try
	{
	  MessageOutputStream mos=tn.getMOS(this,false); 
	  writeTo(mos);
	  mos.send();
	}
      catch(Exception e)
	{
	  e.printStackTrace();
	}
    }
  
  //************************************************************
  
  public void writeTo(DataOutputStream dos) throws Exception
    // called when sending initialization information.
    // here also called to send change information.
    {
      dos.writeInt(x);	// writes x coordinate of the icon
      dos.writeInt(y);	// writes y coordinate of the icon
    };
  public void dispose() {};
  public void receive(DataInputStream dis) throws Exception
    // called when a message arrives
    {
      update(dis);		
    };
  public void update(DataInputStream dis) throws Exception
    // called when update information arrives
    {
      x=dis.readInt();
      y=dis.readInt();
      setBounds(x,y,size,size);
      getParent().repaint();
    };
}

//
// Icon2 draws different shape than Icon1. It also must be public so that TNode class could
// automaticly recreate it on other clients.
//
public class Icon2 extends Icon1	   
{
  public Icon2(TNode tn,Point p) throws Exception
    {
      super(tn,p);
    }
  public Icon2(TNode tn,DataInputStream dis) throws Exception
    {
      super(tn,dis);
    }
  public void paint(Graphics g)
    {
      g.setColor(Color.pink);
      g.fillOval(0,0,size,size);
    }
}

//
// TangoObjectExample creates TNode and buttons for adding icons.
// In order to create TNode TLAgent object has to be created. The TangoObjectExample
// checks if it is connected to the Tango system and if so it creates TLAgentApplet object,
// otherwise it creates TLAgentFake object which can work outside TangoSystem.
//
public class TangoObjectExample extends Applet implements MouseListener
{
  CheckboxGroup cg;  // checkboxgroup for icon selector
  TNode tn;	     // tango service class
  TLAgent ta;
  public void init()
    {
      try
	{
	  try
	    {
	      ta=new TLAgentApplet(this);
	    }
	  catch(Exception e)
	    {
	      System.out.println("Launching fake tango");
	      ta=new webwisdom.tango.fake.TLAgentFake(getParameter("username"),
						      (new Boolean(getParameter("ismaster"))).booleanValue(),
						      getCodeBase().getHost(),
						      (new Integer(getParameter("port"))).intValue());
	    }
	  tn=new TNode(ta,this.getClass().getClassLoader());
	  setLayout(new BorderLayout());
	  // adding radio buttons
	  Panel p0=new Panel();
	  add("North",p0);
	  cg=new CheckboxGroup();
	  p0.add(new Checkbox("Icon1",true,cg));
	  p0.add(new Checkbox("Icon2",false,cg));
	  // adding drawing panel
	  DrawPanel p1=new DrawPanel();
	  p1.addMouseListener(this);
	  add("Center",p1);
	  // draw panel registers in the tango node with name "drawpanel"
	  // this is needed while adding icons on other instances to find their parent
	  tn.add("drawpanel",p1);
	  tn.init();
	  tn.start();
	}
      catch(Exception e)
	{
	  System.out.println("Couldn't initialize any TLAgent");
	  e.printStackTrace();
	}
    }
  public void destroy()
    {
      ta.exit();
      tn.dispose();
      super.destroy();
    }
  
  public void mouseClicked(MouseEvent me){}
  public void mouseEntered(MouseEvent me){}
  public void mouseExited(MouseEvent me){}
  public void mousePressed(MouseEvent me)
    {
      // mouse events from the drawPanel
      try
	{
	  if(me.getID()==MouseEvent.MOUSE_PRESSED)
	    {
	      // depending on current state of radio buttons, creating either
	      // icon1 or Icon2 class objects
	      if(cg.getSelectedCheckbox().getLabel().equals("Icon1"))
		{		 
		  new Icon1(tn,me.getPoint());
		}
	      else
		{
		  new Icon2(tn,me.getPoint());
		}
	    }
	}
      catch(Exception e)
	{
	  e.printStackTrace();
	}
    }
  public void mouseReleased(MouseEvent me){}
}

//Draw panel is sort of the dummy class. It must implement TObject interface
//so that it can be added to TNode
class DrawPanel extends Panel implements TObject
{
  public DrawPanel(){setLayout(null);}
  public void writeTo(DataOutputStream dos) throws Exception {};
  public void dispose() {};
  public void receive(DataInputStream dis) throws Exception {};
  public void update(DataInputStream dis) throws Exception {};
}










